#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>

#include <Carbon/Carbon.h>

#include "config.h"
#include "video_out.h"
#include "video_out_internal.h"

#include "aspect.h"

static vo_info_t info = 
{
	"Mac OSX (Quartz 2D)",
	"quartz",
	"Nicolas Plourde <nicolasplourde@hotmail.com>",
	""
};

LIBVO_EXTERN(quartz)

static unsigned char *ImageData=NULL;

static uint32_t image_width;
static uint32_t image_height;
static uint32_t image_depth;
static uint32_t image_bytes;

static int int_pause;

static int slice_height=1;

WindowRef theWindow; 
CGContextRef context;
CGRect bounds;
CGColorSpaceRef colorspace;

static uint32_t config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_height, uint32_t flags, char *title, uint32_t format)
{
	WindowAttributes  windowAttrs;
	Rect              contentRect; 
	CFStringRef       titleKey;
	CFStringRef       windowTitle; 
	OSStatus          result; 
	
	image_width = width;
	image_height = height;
	image_depth = IMGFMT_RGB_DEPTH(format);
	image_bytes=(IMGFMT_RGB_DEPTH(format)+7)/8;
	int_pause = 0;
	
    vo_dwidth = d_width;
    vo_dheight = d_height;

	aspect_save_orig(width,height);
	aspect_save_prescale(d_width,d_height);
	aspect_save_screenres(image_width,image_height);

	aspect(&d_width,&d_height,A_NOZOOM);

	if ( ImageData ) free( ImageData );
		ImageData=malloc(image_width*image_height*image_bytes);

	windowAttrs =     kWindowCloseBoxAttribute
					| kWindowCollapseBoxAttribute 
					| kWindowStandardHandlerAttribute
					| kWindowInWindowMenuAttribute;

	SetRect(&contentRect, 0,  0, d_width, d_height);
	CreateNewWindow(kDocumentWindowClass, windowAttrs, &contentRect, &theWindow);

	titleKey	= CFSTR("MPlayer");
	windowTitle = CFCopyLocalizedString(titleKey, NULL);
	result		= SetWindowTitleWithCFString(theWindow, windowTitle);
	CFRelease(titleKey);
	CFRelease(windowTitle);
	
	RepositionWindow(theWindow, NULL, kWindowCascadeOnMainScreen);
	ShowWindow (theWindow);
			
	//SetWindowAlpha(theWindow,0.8);
	CreateCGContextForPort(GetWindowPort(theWindow),&context);
	bounds = CGRectMake( 0, 0, d_width, d_height);
	
	colorspace = CGColorSpaceCreateDeviceRGB();
		
	return 0;
}

static void check_events(void)
{
	int noErr;
	EventRef theEvent;
	EventTargetRef theTarget;

	theTarget = GetEventDispatcherTarget();
      
	ReceiveNextEvent(0, NULL,kEventDurationNoWait,true, &theEvent);
	SendEventToEventTarget (theEvent, theTarget);
	ReleaseEvent(theEvent);
		
    //int e=vo_x11_check_events(mDisplay);
    //if(e&VO_EVENT_RESIZE) resize(vo_dwidth,vo_dheight);
    if(VO_EVENT_EXPOSE && int_pause) flip_page();
}

static void draw_osd(void)
{
}

static void flip_page(void)
{
	CGContextFlush(context);
}

static uint32_t draw_slice(uint8_t *src[], int stride[], int w,int h,int x,int y)
{
	return -1;
}

static uint32_t draw_frame(uint8_t *src[])
{
	CGImageRef image;

	CGDataProviderRef dataProviderRef = CGDataProviderCreateWithData(0, src[0], image_width*image_height*image_bytes, 0);

	image = CGImageCreate(
	image_width,
	image_height,
	8,
	image_depth,
	((image_width * image_depth) + 7) / 8,
	colorspace,
	kCGImageAlphaNone,
	dataProviderRef,
	0,
	0,
	kCGRenderingIntentDefault);

	CGContextDrawImage(context, bounds, image);

	return 0; 
}

static uint32_t query_format(uint32_t format)
{
    if ((format == IMGFMT_RGB24))
        return VFCAP_CSP_SUPPORTED;
    return 0;
}


static void uninit(void)
{
}

static uint32_t preinit(const char *arg)
{
    return 0;
}

static uint32_t control(uint32_t request, void *data, ...)
{
  switch (request)
  {
	case VOCTRL_PAUSE: return (int_pause=1);
	case VOCTRL_RESUME: return (int_pause=0);
	case VOCTRL_QUERY_FORMAT:
		return query_format(*((uint32_t*)data));
  }
  return VO_NOTIMPL;
}
